#ifndef DESCRIPTIONS_WINDOW_H
#define DESCRIPTIONS_WINDOW_H

#include <qwidget.h>
#include <qcombobox.h>
#include <qmap.h>
#include <qlistview.h>

#include "gadu.h"

class QDateTimeEdit;
class QLineEdit;
class QPushButton;
class DescHistory;
class QPopupMenu;

class DescHistorySource : public QComboBox
{
	public:
		DescHistorySource(QWidget* parent);
		void insertItem( const QString &t, UinType uin );
		UinType getUin(int index);

	private:
		QMap<int,UinType> uinsMap;
};

class DescHistoryListItem : public QListViewItem
{
	public:
		DescHistoryListItem(QListView* parent);

		void setId(QString id);
		void setAuthorUid(QString authorUid);
		void setMarked(bool m);
		QString getId();
		QString getAuthorUid();
		bool getMarked();
		virtual void setup();
		void paintCell( QPainter * p, const QColorGroup & cg, int column, int width, int align );

	private:
		QString m_id;
		QString m_authorUid;
		bool marked;
};

class DescriptionsWindow : public QWidget
{
	Q_OBJECT

	public:
		DescriptionsWindow(DescHistory* descHistory);
		~DescriptionsWindow();
		void addContact(UserListElement ule);
		void select(int index);
		void select(UinType uin);

	private:
		DescHistorySource* source;
		QDateTimeEdit* fromDate;
		QDateTimeEdit* toDate;
		QLineEdit* filter;
		QPushButton* applyFilter;
		QPushButton* resetFilter;
		QPushButton* close;
		QListView* list;
		DescHistory* descHistoryObj;
		QPopupMenu* menu;

		QString getAuthorByUid(UinType uin);
		void resetValues();

	private slots:
		void refreshDescriptions();
		void activated(int index);
		void doubleClicked(QListViewItem* item, const QPoint &p, int col);
		void applyReset();
		void openDescUrl();
		void copyDesc();
		void openChat();
		void raiseMenu(QListViewItem* it, const QPoint& p, int col);
};

#endif
